/* GStreamer Media Player
 * (c) 2004 Ronald Bultje <rbultje@ronald.bitfreak.net>
 *
 * video.c: video widget.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include <gdk/gdkx.h>
#include <gst/xoverlay/xoverlay.h>

#include "video.h"

static void	gst_player_video_class_init	(GstPlayerVideoClass *klass);
static void	gst_player_video_init		(GstPlayerVideo *video);
static void	gst_player_video_dispose	(GObject        *object);
static void	gst_player_video_realize	(GtkWidget      *widget);
static void	gst_player_video_size_request	(GtkWidget      *widget,
						 GtkRequisition *req);
static void	gst_player_video_size_allocate	(GtkWidget      *widget,
						 GtkAllocation  *alloc);
static gboolean	gst_player_video_expose		(GtkWidget      *widget,
						 GdkEventExpose *event);
static void	gst_player_video_draw		(GstPlayerVideo *video,
						 gboolean        on);

static GtkWidgetClass *parent_class = NULL;

GType
gst_player_video_get_type (void)
{
  static GType gst_player_video_type = 0;

  if (!gst_player_video_type) {
    static const GTypeInfo gst_player_video_info = {
      sizeof (GstPlayerVideoClass),
      NULL,
      NULL,
      (GClassInitFunc) gst_player_video_class_init,
      NULL,
      NULL,
      sizeof (GstPlayerVideo),
      0,
      (GInstanceInitFunc) gst_player_video_init,
      NULL
    };

    gst_player_video_type =
	g_type_register_static (GTK_TYPE_WIDGET, 
				"GstPlayerVideo",
				&gst_player_video_info, 0);
  }

  return gst_player_video_type;
}

static void
gst_player_video_class_init (GstPlayerVideoClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

  parent_class = g_type_class_ref (GTK_TYPE_WIDGET);

  gobject_class->dispose = gst_player_video_dispose;

  widget_class->realize = gst_player_video_realize;
  widget_class->size_allocate = gst_player_video_size_allocate;
  widget_class->size_request = gst_player_video_size_request;
  widget_class->expose_event = gst_player_video_expose;
}

static void
gst_player_video_init (GstPlayerVideo *video)
{
  video->element = NULL;
}

GtkWidget *
gst_player_video_new (GstElement *element)
{
  GstPlayerVideo *video =
      g_object_new (GST_PLAYER_TYPE_VIDEO, NULL);

  if (element) {
    gst_object_ref (GST_OBJECT (element));
    video->element = element;
  }
  
  return GTK_WIDGET (video);
}

static void
gst_player_video_dispose (GObject *object)
{
  GstPlayerVideo *video = GST_PLAYER_VIDEO (object);

  if (video->element) {
    /* shut down overlay... */
    gst_player_video_draw (video, FALSE);

    /* ...and be happy */
    gst_object_unref (GST_OBJECT (video->element));
    video->element = NULL;
  }
}

static void
gst_player_video_realize (GtkWidget *widget)
{
  GdkWindowAttr attributes;
  gint attributes_mask;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GST_PLAYER_IS_VIDEO (widget));

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.event_mask = gtk_widget_get_events (widget) | 
                            GDK_EXPOSURE_MASK |
			    GDK_BUTTON_PRESS_MASK | 
                            GDK_BUTTON_RELEASE_MASK |
			    GDK_POINTER_MOTION_MASK |
                            GDK_POINTER_MOTION_HINT_MASK;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);

  attributes_mask = GDK_WA_X | GDK_WA_Y |
		      GDK_WA_VISUAL | GDK_WA_COLORMAP;
  widget->window = gdk_window_new (widget->parent->window,
				   &attributes, attributes_mask);

  widget->style = gtk_style_attach (widget->style, widget->window);

  gdk_window_set_user_data (widget->window, widget);

  gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
}


static void
gst_player_video_size_request (GtkWidget      *widget,
			       GtkRequisition *requisition)
{
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GST_PLAYER_IS_VIDEO (widget));
  g_return_if_fail (requisition != NULL);

  requisition->width = 160;
  requisition->height = 120;
}

static void
gst_player_video_size_allocate (GtkWidget     *widget,
				GtkAllocation *allocation)
{
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GST_PLAYER_IS_VIDEO(widget));
  g_return_if_fail (allocation != NULL);

  widget->allocation = *allocation;

  if (GTK_WIDGET_REALIZED (widget))
    gdk_window_move_resize (widget->window,
                            allocation->x, allocation->y,
                            allocation->width, allocation->height);
}

static void
gst_player_video_draw (GstPlayerVideo *video,
		       gboolean              on)
{
  XID window;
  GstElement *xoverlay;

  if (GST_IS_BIN (video->element)) {
    xoverlay = gst_bin_get_by_interface (GST_BIN (video->element),
					 GST_TYPE_X_OVERLAY);
  } else {
    xoverlay = video->element;
  }
  if (!GST_IS_X_OVERLAY (xoverlay))
    return;

  if (on) {
    window = GDK_WINDOW_XWINDOW (GTK_WIDGET (video)->window);
  } else {
    window = 0;
  }

  gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (xoverlay), window);
}

static gboolean
gst_player_video_expose (GtkWidget      *widget,
			 GdkEventExpose *event)
{
  GstPlayerVideo *video;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GST_PLAYER_IS_VIDEO (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->count > 0)
    return FALSE;

  video = GST_PLAYER_VIDEO (widget);
 
  gst_player_video_draw (video, TRUE);

  return FALSE;
}
